--[[if not _LCache then
	_LCache = {}
end]]

map = {}
map.__index = map
function map.new(Map_ID)
	if cache and type(cache) == "table" and cache[Map_ID] then
		return cache[Map_ID]
	end
	local p = {}
	setmetatable(p,map)
	p.id = Map_ID
	if cache and type(cache) == "table" then table.insert(cache,Map_ID,p) end
	return p
end
function map.add(Map_ID, X, Y, Z, Name) return map.new(Map_Add(Map_ID, X, Y, Z, Name)) end
function map.all()
	Map_Examine()
	return function ()
		if Map_Next() == 0 then return nil end
		return map.new(Map_Get_ID())
	end
end
function map:name(Name) if Name then return Map_Set_Name(self.id, Name) end return Map_Get_Name(self.id) end

function map:filename(Name) if Name then return Map_Set_Filename(self.id, Name) end return Map_Get_Filename(self.id) end
function map:ofilename(Name) if Name then return Map_Set_Filename_Overview(self.id, Name) end return Map_Get_Filename_Overview(self.id) end
function map:load(Name) return Map_Load(self.id, Filename) end

function map:delete(...) return Map_Delete(self.id, ...) end
function map:resend() return Map_Resend(self.id) end
function map:fill(...) return Map_Fill(self.id,...) end
function map:save(...) return Map_Save(self.id,...) end
function map:osave(...) return Map_Save_Overview(self.id,...) end
function map:size(X,Y,Z) if X and Y and Z then return Map_Resize(self.id, X,Y,Z) end return Map_Get_Dimensions(self.id) end
function map:spawn(X,Y,Z,R,L) if X and Y and Z then return Map_Set_Spawn(self.id, X, Y, Z, R, L) end return Map_Get_Spawn(self.id) end
function map:message(Message) return System_Message_Network_Send_2_All(self.id, Message) end

map.rank = {}
function map.rank:build(Rank) if Rank then return Map_Set_Rank_Build(self.id,Rank) end return Map_Get_Rank_Build(self.id) end
function map.rank:join(Rank) if Rank then return Map_Set_Rank_Join(self.id,Rank) end return Map_Get_Rank_Join(self.id) end
function map.rank:show(Rank) if Rank then return Map_Set_Rank_Show(self.id,Rank) end return Map_Get_Rank_Show(self.id) end

map.block = {} --might be bad idea to use these(due to speed).. but useful anyway
map.block.__index = map.block
setmetatable(map.block,map)
function map.block:fast(P,...)
	print("block change")
	if type(P) == "table" then
		if P.number then P = p.number end
	else
		P = P
	end
	return Map_Block_Change_Fast(P, self.id, ...)
end
function map.block:normal(P,...)
	if type(P) == "table" then
		if P.id then P = p.id end
	else
		P = P
	end
	return Map_Block_Change(P, self.id, ...)
end
function map.block:move(...)
	return Map_Block_Move(self.id, ...)
end
function map.block:rank(X,Y,Z,Rank) if Rank then Map_Block_Set_Rank(self.id, X, Y, Z, Rank) end return Map_Block_Get_Rank(self.id, X, Y, Z) end

player = {}
player.__index = player
function player.new(Player_Number, cache)
	if cache and cache[Player_Number] then
		return cache[Player_Number]
	end
	local p = {}
	setmetatable(p,player)
	p.number = Player_Number
	if cache and type(cache) == "table" then table.insert(cache,Player_Number,p) end
	return p
end
function player.all(cache)
	Player_Examine()
	return function ()
		if Player_Next() == 0 then return nil end
		return player.new(Player_Get_Number(),cache)
	end
end
function player:name() return Player_Get_Name(self.number) end
function player:IP() return Player_Get_IP(self.number) end
function player:rank(R) if R then return Player_Set_Rank(self.number,R) end return Player_Get_Rank(self.number) end
function player:online() return Player_Get_Online(self.number) end
function player:ontime() return Player_Get_Ontime(self.number) end
function player:col() return Rank_Get_Color(self:rank(),0) end
function player:colname() return self:col()..self:name() end
client = {}
client.__index = client

function client.new(Client_ID,cache)
	if cache and cache[Client_ID] then
		return cache[Client_ID]
	end
	local p = {}
	setmetatable(p,client)
	p.id = Client_ID

	p.number = Client_Get_Player_Number(Client_ID)
	p.player = player.new(p.number) --allows easy access to player functions. e.g. client.player:getRank()
	if cache and type(cache) == "table" then table.insert(cache,Client_ID,p) end
	return p
end
function client.addFake(Name,HM, cache) return client.new(Client_Fake_Add(Name, HM),cache) end
function client.all(cache)
	Client_Examine()
	local Client_ID = 0
	return function()
		if Client_Next() == 0 then return nil end
		return client.new(Client_Get_ID(),cache)
	end
end
function client:name() return Client_Get_Name(self.id) end
function client:mapID() return Client_Get_Map_ID(self.id) end --map will return object.. :p
function client:map() return map.new(self:mapID()) end
--function client:getPos() return Client_Get_X(self.id),Client_Get_Y(self.id),Client_Get_Z(self.id) end
function client:rot() return Client_Get_Rotation(self.id),Client_Get_Look(self.id) end
function client:pos(M,X,Y,Z,R,L,SO,P)
	if not M then
		return Client_Get_X(self.id),Client_Get_Y(self.id),Client_Get_Z(self.id)
	end
	--if not P then P = -1 end if not SO then SO = 1 end if not L then L = 0 end if not R then R = 0 end
	return Client_Position_Set(self.id, M, X, Y, Z, R, L, SO, P)
end
function client:move(X,Y,Z,S) return Client_Move_Interpolated(self.id, X, Y, Z, S) end
function client:IP() return Client_Get_IP(self.id) end
function client:ip() return self:IP() end
function client:kick(Message,Hide) return Client_Kick(self.id, Message, Hide) end
function client:kill() return Client_Kill(self.id) end
function client:hideMode(HM) if HM then return Client_Hidemode_Set(self.id, HM) end return Client_Hidemode_Get(self.id) end
function client:message(M) return System_Message_Network_Send(self.id, M) end
function client:rank(R) return self.player:rank(R) end
function client:online() return self.player:getOnline() end
function client:ontime() return self.player:getOntime() end
function client:colname() return self.player:colname() end

function client.find(name,cs,num)
	if not num then num = 1 end
	if not cs then cs = false end
	local snum = num
	if cs then name = name:lower() end
	for client in client.all() do
		if (cs and client:name():lower() == name) or client:name() == name then
			num = num-1
			if num == 0 then
				return client
			end
		end
	end
	if snum == num then
		return -1
	else
		return -2
	end
end


--[[
rank = {}
rank.__index = map
function rank.new(Rank,Exact)
	local r = {}
	setmetatable(r,player)
	r.rank = Rank
	r.exact = Exact
	return r
end
function rank.add(Rank, Name, Col) return rank.new(Rank_Add(Rank, Name, Col)) end
function rank.all()
	Rank_Examine()
	return function ()
		if Rank_Next() == 0 then return nil end
		return rank.new(Rank_Get_Rank())
	end
end
function rank:name() return Rank_Get_Name(self.rank,self.exact) end
function rank:colour() return Rank_Get_Colour(self.rank,self.exact) end
function rank:color() return self:colour() end
function rank:root() return Rank_Get_Root(self.rank,self.exact) end]] --cant see any point
--print(client.find("YLT",true,2):name())